#if !defined(__PARAMETERS_H)
#define __PARAMETERS_H

#include <hair/common.h>

/**
 * This data structure stores all model-relevant parameters, as well
 * as file paths used as input and output during the different stages
 */
struct Parameters {
	inline Parameters() : cosTheta(NULL), sinTheta(NULL) { }

	/* Parse parameters from the command line or an 
	 * external file specified on the command line */
	Parameters(int argc, char **argv);

	/* Free all memory */
	~Parameters();

	/* To be called once after the initialization */
	void precompute();

	/* Return a string representation */
	std::string toString() const;

	/* Was the program invoked using the image filtering mode? */
	bool filter;
	/* Input filename format (16 bit, 1 channel) */
	std::string input;
	/* Output filename format for filtered data */
	std::string filterOutput;
	/* Number of gabor filters used to estimate orientation - default: 16 */
	int gaborCount;
	/* Lower bound for hysteresis thresholding - default: 30 */
	int hysteresisLow;
	/* Upper bound for hysteresis thresholding - default: 100 */
	int hysteresisHigh;
	/* Extents of the gaussian pre-filtering pass - default: 1 (inactive) */
	int gaussianExtents;
	/* Number of captured angles - default: 12 */
	int numAngles;
	/* Number of captured depth - default: 400 */
	int numDepths;	
	/* Initial translation offset in world-space units - default: -50 */
	float translateStart;
	/* Final translation offset in world-space units - default: 50*/
	float translateEnd;
	/* Field of view in degrees - default: 11 */
	float xfov;
	/* Distance to the focus plane - default: 278 */
	float fPlaneDist;
	/* Per-color X resolution on the camera sensor - default: 768 */
	int xres;
	/* Per-color Y resolution on the camera sensor - default: 768 */
	int yres;
	/* Projected X coordinate of the rotation axis */
	float centerX;
	/* Filter offset */
	float filterOffset;
	/* Thread count during reconstruction */
	int threadCount;
	/* Growing algorithm: max. tangential correction offset in pixels */
	float correctionCutoff;
	/* Growing algorithm: maximum angle to the previous fitted direction (in degrees) */
	float directionCutoff;
	/* Required number of observations for the direction/correction fitting algorithms */
	int minObservations;
	/* Sizes of the oriented box query */
	Vector3 sheetSize;
	Float angleThreshold;

	/*==========================*/
	/*  Precomputed parameters  */
	/*==========================*/
	float f;
	float translateStep, absTranslateStep, angleStep;
	float pixelWidth;
	int centerDepth;
	float aspectZ, absAspectZ;
	float *cosTheta, *sinTheta, *filterOrientations, *ridgeOrientations;
};

#endif /* __PARAMETERS_H */
