#if !defined(__WORKERTHREAD_H)
#define __WORKERTHREAD_H

#include <hair/datamodel.h>

class WorkerThread : public QThread {
	Q_OBJECT
public:
	WorkerThread(const AABB &aabb, DataModel *dataModel);
	~WorkerThread();
	
	enum EReturnCode {
		Failure = 0,
		Success = 1,
		Done = 2
	};

    inline const AABB &getSeedAABB() const { return m_seedAABB; }
    inline AABB &getSeedAABB() { return m_seedAABB; }
    inline const ActiveSet &getActiveSet(int idx) const { return m_activeSet[idx]; }
    inline ActiveSet &getActiveSet(int idx) { return m_activeSet[idx]; }
    inline bool isActive() const { return m_active; }
    inline void stop() { m_active = false; }

	virtual void run();

	/* Search for seed point used to start growing a fiber */
	EReturnCode findSeed(ActiveSet &activeSet);

	/* Perform one growth step. The paramenter is set to true 
	   when the 'Grow feature' button was pressed */
	bool grow(bool onlyOneStep);
private:
	DataModel *m_dataModel;
	ActiveSet m_activeSet[2];
	AABB m_aabb[2], m_seedAABB;
	int m_side;
	PixelSet m_previousSeeds;
	uint16_t *m_cells;
	bool m_active, m_growing, m_regenerateCells;
	std::vector<KDTree::Observation *> m_result;
};

#endif /* __WORKERTHREAD_H */
